#!/bin/bash

. "$MKIT_DIR/include/ini.sh"   || die "cannot import ini.sh"
. "$MKIT_DIR/include/facts.sh" || die "cannot import facts.sh"

_bump_version() {
    #
    # Bump version on stdin by level $1 (x, y or z)
    #
    local rlevel=$1     # release level
    local old           # old version
    local oldx          # ... X
    local oldz          # ... Z
    local oldy          # ... Y
    local tmpy          # Y parsing cache
    local new           # bumped version
    read -r old
    oldx=${old%.*.*}
    oldz=${old#*.*.}
    tmpy=${old%.*}
    oldy=${tmpy#*.}
    case $rlevel in
        x) new="$((oldx+MKIT_BUMPSIZE)).0.0"            ;;
        y) new="$oldx.$((oldy+MKIT_BUMPSIZE)).0"        ;;
        z) new="$oldx.$oldy.$((oldz+MKIT_BUMPSIZE))"    ;;
        *) die "invalid release level: $1"  ;;
    esac
    echo "$new"
}

_relck() {
    #
    # Die if blocking condition $1 is detected
    #
    local condition=$1      # condition name
    local oracle            # expected value
    case "$condition" in
        git_present)
            git rev-parse HEAD >&/dev/null\
             || die "cannot do this outside git repo"
            ;;
        at_relsrc)
            oracle=$(ini 1value project:relsrc)
            git_fact current_branch \
              | grep -qFx "$oracle" \
             || die "you are not on release source branch: $oracle"
            ;;
        not_dirty)
            git diff --shortstat 2>/dev/null \
              | grep -q . \
             && die "tree is dirty!"
            ;;
        tags_ok)
            git_fact latest_tag \
              | grep -q . \
             || die "cannot find latest tag"
            ;;
        vbump_hot)
            git diff-tree --no-commit-id --name-only -r HEAD \
              | grep -qFx mkit.ini \
             || die "last change must be version bump in mkit.ini"
            ;;
        no_wip)
            git_fact reldiff \
              | grep '^....... WIP ' \
             && die "WIP commit since $(git_fact latest_tag)"
            ;;
        ini_version)
            oracle=$(git_fact latest_version | _bump_version "$rlevel")
            ini 1value project:version  \
              | grep -qFx "$oracle" \
             || die "new version not in mkit.ini: $oracle"
            ;;
        *)
            die "unknown release check: $condition"
            ;;
    esac
}

_release() {
    #
    # Prepare release
    #
    # Span release routines: make a signed tag, check branch
    # and update release branch
    #
    # FIXME: Using project:prerl as build.sh:semver() does may not be
    #        compatible with this release strategy
    #
    local rlevel=$1     # release level (x, y or z)
    local newtag        # new tag
    local reldst        # release destination branch (if any)

    _relck git_present
    _relck at_relsrc
    _relck not_dirty
    _relck tags_ok
    _relck vbump_hot
    _relck no_wip
    _relck ini_version

    newtag=$(git_fact latest_version | _bump_version "$rlevel" | git_ver2tag )
    set -e
    debug_var newtag
    $MKIT_DRY && return
    git tag -m "$(_release_msg)" "$newtag"

    reldst=$(ini 1value project:reldst)
    debug_var reldst
    if test -n "$reldst"; then
        git branch -f "$reldst" "$newtag"
    fi
}

_release_msg() {
    #
    # Generate message for annotated tag
    #
    # The last commit before issuing a release must be "Bump version" commit
    # suggested by _vbump_gitmsg() and  manually edited by user.  Since the
    # commit contains changelog, this function just uses the message body.
    #
    # The sort message (first line) is replaced with a nicer one (with project
    # name, codename and version).
    #
    echo "$(ini 1value project:name) $newtag - $(ini 1value project:codename)"
    echo
    git show -s --format=%B \
      | tail -n +3
}

_vbump() {
    #
    # Do version bump at level $1
    #
    # Perform checks, compute new version, update mkit.ini and initiate
    # 'Bump version' commit with changelog template.
    #
    local rlevel=$1     # bump level (x, y or z)
    local nextver       # version after the bump
    _relck git_present
    _relck at_relsrc
    _relck not_dirty
    nextver=$(ini 1value project:version | _bump_version "$rlevel")
    debug_var nextver
    $MKIT_DRY && return
    update_version "$nextver" mkit.ini \
      || die "failed to update version in mkit.ini"
    git add mkit.ini \
      || die "failed to add mkit.ini to the index"
    git commit -e -m "$(_vbump_gitmsg)"
}

_vbump_gitmsg() {
    #
    # Compose git message template for 'Bump version' commit
    #
    echo "Bump version"
    echo ""
    echo "Overview of changes:"
    echo ""
    git_fact reldiff \
      | sed '
            s/^[a-f0-9]\{7\} / *  &/; t PATHS
            s/^/        /
            :PATHS
        '
}

release_x() {
    #
    # Perform release on X level
    #
    _release x
}

release_y() {
    #
    # Perform release on Y level
    #
    _release y
}

release_z() {
    #
    # Perform release on Z level
    #
    _release z
}

vbump_x() {
    #
    # Perform version bump on X level
    #
    _vbump x
}

vbump_y() {
    #
    # Perform version bump on Y level
    #
    _vbump y
}

vbump_z() {
    #
    # Perform version bump on Z level
    #
    _vbump z
}
